#!/usr/bin/env python
# coding: utf-8

#importing all the libraries
import os
import ntpath
import shutil
import tempfile
import atexit
import argparse
import numpy as np
import skimage.io as io
import skimage.transform as trans
from Aug_n_Train import *
import tensorflow as tf
from keras.models import *
from keras.layers import *
from keras.optimizers import *
from keras.callbacks import ModelCheckpoint, LearningRateScheduler
from keras import backend as keras
import matplotlib
import matplotlib.pyplot as plt
from optparse import OptionParser
from astropy.io import fits
from astropy.utils.data import get_pkg_data_filename
import os.path
from PIL import Image




# Unet convolutional neural network (Mathematical Model)
def unet(pretrained_weights = None, input_size = (256,256,1), filters = None ,kernel_size=None,output_channels=None):

    inputs = Input(input_size)
    conv1 = Conv2D(2**filters, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(inputs)
    conv1 = Conv2D(2**filters, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv1)
    pool1 = AveragePooling2D(pool_size=(2,2))(conv1)
    conv2 = Conv2D(2**(filters+1), kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(pool1)
    conv2 = Conv2D(2**(filters+1), kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv2)
    pool2 = AveragePooling2D(pool_size=(2,2))(conv2)
    # conv3 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(pool2)
    # conv3 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv3)
    # pool3 = AveragePooling2D(pool_size=(2,2))(conv3)
    # conv4 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(pool2)
    # conv4 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv4)
    # #drop4 = Dropout(0.5)(conv4)
    # pool4 = AveragePooling2D(pool_size=(2,2))(conv4)

    conv5 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(pool2)
    conv5 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv5)
    #drop5 = Dropout(0.5)(conv5)

    # up6 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(UpSampling2D(size = (2,2))(conv5))
    # merge6 = concatenate([conv4,up6], axis = 3)
    # conv6 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(merge6)
    # conv6 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv6)

    # up7 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(UpSampling2D(size = (2,2))(conv6))
    # merge7 = concatenate([conv3,up7], axis = 3)
    # conv7 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(merge7)
    # conv7 = Conv2D(2**filters+2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv7)

    up8 = Conv2D(2**filters+1, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(UpSampling2D(size = (2,2))(conv5))
    merge8 = concatenate([conv2,up8], axis = 3)
    conv8 = Conv2D(2**filters+1, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(merge8)
    conv8 = Conv2D(2**filters+1, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv8)

    up9 = Conv2D(2**filters, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(UpSampling2D(size = (2,2))(conv8))
    merge9 = concatenate([conv1,up9], axis = 3)
    conv9 = Conv2D(2**filters, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(merge9)
    conv9 = Conv2D(2**filters, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv9)
    conv9 = Conv2D(2, kernel_size,activation = 'relu', padding = 'same', kernel_initializer = 'he_normal')(conv9)
    conv10 = Conv2D(1, (1,1), activation = 'sigmoid')(conv9)

    model = Model(input = inputs, output = conv10)

    if(pretrained_weights):
    	model.load_weights(pretrained_weights)
    return model


#This is a parser of arguments for the different folders

class readable_dir(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        prospective_dir=values
        if not os.path.isdir(prospective_dir):
            raise argparse.ArgumentTypeError("readable_dir:{0} is not a valid path".format(prospective_dir))
        if os.access(prospective_dir, os.R_OK):
            setattr(namespace,self.dest,prospective_dir)
        else:
            raise argparse.ArgumentTypeError("readable_dir:{0} is not a readable dir".format(prospective_dir))

# This is a function to parse argument via run time
def get_options():

    #ldir1 = tempfile.mkdtemp()
    # ldir = "ATLBS_IMAGES/ExampleTest/"
    # ldir1 = "True"
    # ldir2 = "Mask"
    # ldir3 = "ATLBS_IMAGES/ExampleTest/TrueTest"

    ldir = tempfile.mkdtemp()
    ldir1 = tempfile.mkdtemp()
    ldir2 = tempfile.mkdtemp()
    ldir3 = tempfile.mkdtemp()

    atexit.register(lambda dir=ldir: shutil.rmtree(ldir))
    atexit.register(lambda dir=ldir1: shutil.rmtree(ldir1))
    atexit.register(lambda dir=ldir2: shutil.rmtree(ldir2))
    atexit.register(lambda dir=ldir3: shutil.rmtree(ldir3))


    parser = argparse.ArgumentParser(description="Insert the  path containing training folders,the image and mask folders for training  and the image folder for testing")
    parser.add_argument("-v", "--verbose", action="store_true")
    parser.add_argument("-q", "--quiet", action="store_true")
    parser.add_argument("-p", "--path_to_folders", action=readable_dir, default=ldir,help="path to train and test folders")
    parser.add_argument("-i", "--imageTrain_directory", action=readable_dir, default=ldir1,help="path of images for training ")
    parser.add_argument("-m", "--maskTrain_directory", action=readable_dir, default=ldir2,help="path of corresponding masks for training ")
    parser.add_argument("-t", "--imageTest_directory", action=readable_dir, default=ldir3,help="path of images  for testing ")
    parser.add_argument("-f","--filename", help="name of .npy file to store pixel values of predicted images")
    args = parser.parse_args()

    if args.path_to_folders:
        print("1.You have inserted {} as the path to training folders".format(args.path_to_folders))
    if args.imageTrain_directory:
        print("2.You have inserted {} as the image folder for training".format(args.imageTrain_directory))
    if args.maskTrain_directory:
        print("3.You have inserted {} as the mask folder for training".format(args.maskTrain_directory))
    if args.imageTest_directory:
        print("4.You have inserted {} as the image folder for testing".format(args.imageTest_directory))
    if args.filename:
        print("5.You have inserted {} as the file  to save  prediction weights".format(args.filename))
    else:
        print("An error occurred in reading your file!")

    return  args

# a function to create the test generator
def testGenerator(test_path,num_image =30,target_size = (256,256),flag_multi_class=False):
    args = get_options()
    imfolder2=sorted(glob.glob("{}/*.png".format(args.imageTest_directory)))
    for image in imfolder2:
        img = io.imread(image)
        img = trans.resize(img,target_size)
        img = np.reshape(img,img.shape+(1,)) if (not flag_multi_class) else img
        img = np.reshape(img,(1,)+img.shape)
        yield img



def path_leaf(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)

def trainGenerator(batch_size,train_path,image_folder,mask_folder,aug_dict,image_color_mode = "grayscale",
                   mask_color_mode = "grayscale",image_save_prefix  = "image",mask_save_prefix  = "mask",
                   flag_multi_class = False,num_class = 2,save_to_dir = None,target_size = (256,256),seed = 1):

    #ipdb.set_trace()
    '''
    can generate image and mask at the same time
    use the same seed for image_datagen and mask_datagen to ensure the transformation for image and mask is the same
    if you want to visualize the results of generator, set save_to_dir = "your path"
    '''
    # we create two instances with the same arguments
    # Provide the same seed and keyword arguments to the fit and flow methods
    image_datagen = ImageDataGenerator(**aug_dict)
    mask_datagen = ImageDataGenerator(**aug_dict)
    #val_datagen = ImageDataGenerator(**aug_dict)


#     import ipdb; ipdb.set_trace()


    image_generator = image_datagen.flow_from_directory(
        train_path,
        classes = [image_folder],
        class_mode = None,
        color_mode = image_color_mode,
        target_size = target_size,
        batch_size = batch_size,
        save_to_dir = save_to_dir,
        save_prefix  = image_save_prefix,
        seed = seed)

    mask_generator = mask_datagen.flow_from_directory(
        train_path,
        classes = [mask_folder],
        class_mode = None,
        color_mode = mask_color_mode,
        target_size = target_size,
        batch_size = batch_size,
        save_to_dir = save_to_dir,
        save_prefix  = mask_save_prefix,
        seed = seed)



    '''for (img,mask) in val_generator:
        img,mask = adjustData(img,mask)
        print("passed the stage")
        yield val_image_generator.__next__(), val_mask_generator.__next__()'''


    while True:
        try:
            yield image_generator.__next__(), mask_generator.__next__()
        except:
            break
class DisplayCallback(tf.keras.callbacks.Callback):
  def on_epoch_end(self, epoch, logs=None):
    clear_output(wait=True)
    show_predictions()
    print ('\nSample Prediction after epoch {}\n'.format(epoch+1))



# # This for visualizing the batch
# num_batch = 1
# for i,batch in enumerate(myGene):
#     print (i)
#     if(i > num_batch):
#         break


# # summarize history for accuracy
# plt.plot(history.history['accuracy'])
# #plt.plot(history.history['val_acc'])
# plt.title('Model accuracy')
# plt.ylabel('accuracy')
# plt.xlabel('epoch')
# plt.legend(['train', 'test'], loc='upper left')
# plt.savefig('accuracy.png')
#
#
# # summarize history for loss
# plt.plot(history.history['loss'])
# #plt.plot(history.history['val_loss'])
# plt.title('Model loss')
# plt.ylabel('loss')
# plt.xlabel('epoch')
# plt.legend(['train', 'test'], loc='upper left')
# plt.savefig('loss.png')
#
#
#
# for i in range(0,3):
#     d = data[i,:,:,0]
#     BoxPlot=np.quantile(d,([0,0.25,0.5,0.75,1]))
#     print('BoxPlot',BoxPlot,'Q3=',BoxPlot[1])
#     hdu = fits.PrimaryHDU(d)
#     hdul = fits.HDUList([hdu])
#     if os.path.isfile("%dm1g.fits"%i):
#         print ("File exist")
#         os.remove("%dm1g.fits"%i)
#     else:
#         print ("File not exist")
#
#     hdul.writeto("%dm1g.fits"%i)
